@@ -1,3 +1,4 @@
+import logging
 from datetime import timedelta
 from urllib.parse import parse_qsl, urlparse
 
@@ -7,7 +8,7 @@ from django.core.exceptions import ImproperlyConfigured
 from django.db import models, transaction
 from django.urls import reverse
 from django.utils import timezone
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 
 from .generators import generate_client_id, generate_client_secret
 from .scopes import get_scopes_backend
@@ -15,6 +16,9 @@ from .settings import oauth2_settings
 from .validators import RedirectURIValidator, WildcardSet
 
 
+logger = logging.getLogger(__name__)
+
+
 class AbstractApplication(models.Model):
     """
     An Application instance represents a Client on the Authorization server.
@@ -199,7 +203,16 @@ class AbstractGrant(models.Model):
                       :data:`settings.AUTHORIZATION_CODE_EXPIRE_SECONDS`
     * :attr:`redirect_uri` Self explained
     * :attr:`scope` Required scopes, optional
+    * :attr:`code_challenge` PKCE code challenge
+    * :attr:`code_challenge_method` PKCE code challenge transform algorithm
     """
+    CODE_CHALLENGE_PLAIN = "plain"
+    CODE_CHALLENGE_S256 = "S256"
+    CODE_CHALLENGE_METHODS = (
+        (CODE_CHALLENGE_PLAIN, "plain"),
+        (CODE_CHALLENGE_S256, "S256")
+    )
+
     id = models.BigAutoField(primary_key=True)
     user = models.ForeignKey(
         settings.AUTH_USER_MODEL, on_delete=models.CASCADE,
@@ -216,6 +229,10 @@ class AbstractGrant(models.Model):
     created = models.DateTimeField(auto_now_add=True)
     updated = models.DateTimeField(auto_now=True)
 
+    code_challenge = models.CharField(max_length=128, blank=True, default="")
+    code_challenge_method = models.CharField(
+        max_length=10, blank=True, default="", choices=CODE_CHALLENGE_METHODS)
+
     def is_expired(self):
         """
         Check token expiration with timezone awareness
@@ -247,7 +264,7 @@ class AbstractAccessToken(models.Model):
 
     Fields:
 
-    * :attr:`user` The Django user representing resources' owner
+    * :attr:`user` The Django user representing resources" owner
     * :attr:`source_refresh_token` If from a refresh, the consumed RefeshToken
     * :attr:`token` Access token
     * :attr:`application` Application instance
@@ -307,7 +324,7 @@ class AbstractAccessToken(models.Model):
 
     def revoke(self):
         """
-        Convenience method to uniform tokens' interface, for now
+        Convenience method to uniform tokens" interface, for now
         simply remove this token from the database in order to revoke it.
         """
         self.delete()
@@ -340,7 +357,7 @@ class AbstractRefreshToken(models.Model):
 
     Fields:
 
-    * :attr:`user` The Django user representing resources' owner
+    * :attr:`user` The Django user representing resources" owner
     * :attr:`token` Token value
     * :attr:`application` Application instance
     * :attr:`access_token` AccessToken instance this refresh token is
@@ -377,7 +394,10 @@ class AbstractRefreshToken(models.Model):
             if not self:
                 return
 
-            access_token_model.objects.get(id=self.access_token_id).revoke()
+            try:
+                access_token_model.objects.get(id=self.access_token_id).revoke()
+            except access_token_model.DoesNotExist:
+                pass
             self.access_token = None
             self.revoked = timezone.now()
             self.save()
@@ -433,7 +453,30 @@ def clear_expired():
 
     with transaction.atomic():
         if refresh_expire_at:
-            refresh_token_model.objects.filter(revoked__lt=refresh_expire_at).delete()
-            refresh_token_model.objects.filter(access_token__expires__lt=refresh_expire_at).delete()
-        access_token_model.objects.filter(refresh_token__isnull=True, expires__lt=now).delete()
-        grant_model.objects.filter(expires__lt=now).delete()
+            revoked = refresh_token_model.objects.filter(
+                revoked__lt=refresh_expire_at,
+            )
+            expired = refresh_token_model.objects.filter(
+                access_token__expires__lt=refresh_expire_at,
+            )
+
+            logger.info("%s Revoked refresh tokens to be deleted", revoked.count())
+            logger.info("%s Expired refresh tokens to be deleted", expired.count())
+
+            revoked.delete()
+            expired.delete()
+        else:
+            logger.info("refresh_expire_at is %s. No refresh tokens deleted.",
+                        refresh_expire_at)
+
+        access_tokens = access_token_model.objects.filter(
+            refresh_token__isnull=True,
+            expires__lt=now
+        )
+        grants = grant_model.objects.filter(expires__lt=now)
+
+        logger.info("%s Expired access tokens to be deleted", access_tokens.count())
+        logger.info("%s Expired grant tokens to be deleted", grants.count())
+
+        access_tokens.delete()
+        grants.delete()
