@@ -13,7 +13,7 @@ from django.db import transaction
 from django.db.models import Q
 from django.utils import timezone
 from django.utils.timezone import make_aware
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 from oauthlib.oauth2 import RequestValidator
 
 from .exceptions import FatalClientError
@@ -332,20 +332,14 @@ class OAuth2Validator(RequestValidator):
             scope = content.get("scope", "")
             expires = make_aware(expires)
 
-            try:
-                access_token = AccessToken.objects.select_related("application", "user").get(token=token)
-            except AccessToken.DoesNotExist:
-                access_token = AccessToken.objects.create(
-                    token=token,
-                    user=user,
-                    application=None,
-                    scope=scope,
-                    expires=expires
-                )
-            else:
-                access_token.expires = expires
-                access_token.scope = scope
-                access_token.save()
+            access_token, _created = AccessToken.objects.update_or_create(
+                token=token,
+                defaults={
+                    "user": user,
+                    "application": None,
+                    "scope": scope,
+                    "expires": expires,
+                })
 
             return access_token
 
@@ -362,27 +356,11 @@ class OAuth2Validator(RequestValidator):
 
         try:
             access_token = AccessToken.objects.select_related("application", "user").get(token=token)
-            # if there is a token but invalid then look up the token
-            if introspection_url and (introspection_token or introspection_credentials):
-                if not access_token.is_valid(scopes):
-                    access_token = self._get_token_from_authentication_server(
-                        token,
-                        introspection_url,
-                        introspection_token,
-                        introspection_credentials
-                    )
-            if access_token and access_token.is_valid(scopes):
-                request.client = access_token.application
-                request.user = access_token.user
-                request.scopes = scopes
-
-                # this is needed by django rest framework
-                request.access_token = access_token
-                return True
-            self._set_oauth2_error_on_request(request, access_token, scopes)
-            return False
         except AccessToken.DoesNotExist:
-            # there is no initial token, look up the token
+            access_token = None
+
+        # if there is no token or it's invalid then introspect the token if there's an external OAuth server
+        if not access_token or not access_token.is_valid(scopes):
             if introspection_url and (introspection_token or introspection_credentials):
                 access_token = self._get_token_from_authentication_server(
                     token,
@@ -390,15 +368,17 @@ class OAuth2Validator(RequestValidator):
                     introspection_token,
                     introspection_credentials
                 )
-                if access_token and access_token.is_valid(scopes):
-                    request.client = access_token.application
-                    request.user = access_token.user
-                    request.scopes = scopes
 
-                    # this is needed by django rest framework
-                    request.access_token = access_token
-                    return True
-            self._set_oauth2_error_on_request(request, None, scopes)
+        if access_token and access_token.is_valid(scopes):
+            request.client = access_token.application
+            request.user = access_token.user
+            request.scopes = scopes
+
+            # this is needed by django rest framework
+            request.access_token = access_token
+            return True
+        else:
+            self._set_oauth2_error_on_request(request, access_token, scopes)
             return False
 
     def validate_code(self, client_id, code, client, request, *args, **kwargs):
@@ -446,13 +426,38 @@ class OAuth2Validator(RequestValidator):
     def validate_redirect_uri(self, client_id, redirect_uri, request, *args, **kwargs):
         return request.client.redirect_uri_allowed(redirect_uri)
 
+    def is_pkce_required(self, client_id, request):
+        """
+        Enables or disables PKCE verification.
+
+        Uses the setting PKCE_REQUIRED, which can be either a bool or a callable that
+        receives the client id and returns a bool.
+        """
+        if callable(oauth2_settings.PKCE_REQUIRED):
+            return oauth2_settings.PKCE_REQUIRED(client_id)
+        return oauth2_settings.PKCE_REQUIRED
+
+    def get_code_challenge(self, code, request):
+        grant = Grant.objects.get(code=code, application=request.client)
+        return grant.code_challenge or None
+
+    def get_code_challenge_method(self, code, request):
+        grant = Grant.objects.get(code=code, application=request.client)
+        return grant.code_challenge_method or None
+
     def save_authorization_code(self, client_id, code, request, *args, **kwargs):
         expires = timezone.now() + timedelta(
             seconds=oauth2_settings.AUTHORIZATION_CODE_EXPIRE_SECONDS)
-        g = Grant(application=request.client, user=request.user, code=code["code"],
-                  expires=expires, redirect_uri=request.redirect_uri,
-                  scope=" ".join(request.scopes))
-        g.save()
+        Grant.objects.create(
+            application=request.client,
+            user=request.user,
+            code=code["code"],
+            expires=expires,
+            redirect_uri=request.redirect_uri,
+            scope=" ".join(request.scopes),
+            code_challenge=request.code_challenge or "",
+            code_challenge_method=request.code_challenge_method or ""
+        )
 
     def rotate_refresh_token(self, request):
         """
@@ -472,7 +477,11 @@ class OAuth2Validator(RequestValidator):
         if "scope" not in token:
             raise FatalClientError("Failed to renew access token: missing scope")
 
-        expires = timezone.now() + timedelta(seconds=oauth2_settings.ACCESS_TOKEN_EXPIRE_SECONDS)
+        # expires_in is passed to Server on initialization
+        # custom server class can have logic to override this
+        expires = timezone.now() + timedelta(seconds=token.get(
+            "expires_in", oauth2_settings.ACCESS_TOKEN_EXPIRE_SECONDS,
+        ))
 
         if request.grant_type == "client_credentials":
             request.user = None
@@ -508,6 +517,14 @@ class OAuth2Validator(RequestValidator):
             else:
                 # revoke existing tokens if possible to allow reuse of grant
                 if isinstance(refresh_token_instance, RefreshToken):
+                    # First, to ensure we don't have concurrency issues, we refresh the refresh token
+                    # from the db while acquiring a lock on it
+                    # We also put it in the "request cache"
+                    refresh_token_instance = RefreshToken.objects.select_for_update().get(
+                        id=refresh_token_instance.id
+                    )
+                    request.refresh_token_instance = refresh_token_instance
+
                     previous_access_token = AccessToken.objects.filter(
                         source_refresh_token=refresh_token_instance
                     ).first()
@@ -531,28 +548,22 @@ class OAuth2Validator(RequestValidator):
                         source_refresh_token=refresh_token_instance,
                     )
 
-                    refresh_token = RefreshToken(
-                        user=request.user,
-                        token=refresh_token_code,
-                        application=request.client,
-                        access_token=access_token
-                    )
-                    refresh_token.save()
+                    self._create_refresh_token(request, refresh_token_code, access_token)
                 else:
                     # make sure that the token data we're returning matches
                     # the existing token
                     token["access_token"] = previous_access_token.token
+                    token["refresh_token"] = RefreshToken.objects.filter(
+                        access_token=previous_access_token
+                    ).first().token
                     token["scope"] = previous_access_token.scope
 
         # No refresh token should be created, just access token
         else:
             self._create_access_token(expires, request, token)
 
-        # TODO: check out a more reliable way to communicate expire time to oauthlib
-        token["expires_in"] = oauth2_settings.ACCESS_TOKEN_EXPIRE_SECONDS
-
     def _create_access_token(self, expires, request, token, source_refresh_token=None):
-        access_token = AccessToken(
+        return AccessToken.objects.create(
             user=request.user,
             scope=token["scope"],
             expires=expires,
@@ -560,8 +571,14 @@ class OAuth2Validator(RequestValidator):
             application=request.client,
             source_refresh_token=source_refresh_token,
         )
-        access_token.save()
-        return access_token
+
+    def _create_refresh_token(self, request, refresh_token_code, access_token):
+        return RefreshToken.objects.create(
+            user=request.user,
+            token=refresh_token_code,
+            application=request.client,
+            access_token=access_token
+        )
 
     def revoke_token(self, token, token_type_hint, request, *args, **kwargs):
         """
@@ -591,7 +608,7 @@ class OAuth2Validator(RequestValidator):
         """
         Check username and password correspond to a valid and active User
         """
-        u = authenticate(username=username, password=password)
+        u = authenticate(request, username=username, password=password)
         if u is not None and u.is_active:
             request.user = u
             return True
